require File.join(File.dirname(__FILE__), 'lib/certapi')
require File.join(File.dirname(__FILE__), 'lib/certificate_logger')
require File.join(File.dirname(__FILE__), 'lib/certificate_helpers')

module Certificates
  class Main < Sinatra::Base
    set :root, $path

    helpers CertificateHelpers

    configure do
      settings = YAML::load(IO.read('/opt/puppet/share/puppet-dashboard/config/settings.yml'))
      path = '/opt/puppet/share/puppet-dashboard/certs'
      CertAPI.config({
        :ca_location    => settings['ca_server'],
        :ca_port        => settings['ca_port'],
        :ca_file_path   => File.join(path, 'pe-internal-dashboard.ca_cert.pem'),
        :cert_path      => File.join(path, 'pe-internal-dashboard.cert.pem'),
        :cert_key_path  => File.join(path, 'pe-internal-dashboard.private_key.pem'),
        })
      mime_type :csr,
        "application/vnd.puppetlabs.certificate-signing-request.metadata.v1+json"
    end

    before do
      cache_control :no_cache
      @logger = CertificateLogger.new
    end

    def csr_exists?(certname)
      current_cert_status = CertAPI.cert_info certname
      (! [*current_cert_status].empty?) and current_cert_status['state'] == 'requested'
    end

    get '/api/v1/csrs' do
      content_type :csr
      real = CertAPI.csrs
      cached = get_all_cached_nodes
      all = real.concat(cached.values).sort_by { |cert| cert['name'] }
      all.to_json
    end

    get '/api/v1/csr_count' do
      content_type :json
      {:request_count => CertAPI.csrs.count}.to_json
    end

    get '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 404, "#{certname} is not a valid certname. The valid certnames are #{CertAPI.certnames.to_json}"
        # It is okay to divulge a list of certnames in the error message, as
        # the user accessing this information must be authenticated to reach
        # this API. If they have permission access to the API, then they have
        # access to this information anyway.
      else
        content_type :csr
        CertAPI.cert_info(certname).to_json
      end
    end

    # sign a CSR
    put '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 403, "#{certname} could not be accepted because its certificate " +
                  "request has been moved or modified."
        # This status code could be 403 or 404, but since we are providing the
        # reason for the error it is better left as 403.
      else
        api_response = @logger.log_action(user(env), certname, "Add") do
          CertAPI.add(certname)
        end
        unless is_successful?(api_response.code)
          halt 403, "A pending CSR for #{certname} was found, but the CA failed to sign it." +
                    " The CA responded with the message '#{api_response.body}'"
        end
        add_to_cache(certname, CertAPI.cert_info(certname))
        api_response.code
      end
    end

    # sign multiple CSRs
    post '/api/v1/csrs/multi_accept' do
      content_type :json
      result = {}
      params[:names].each do |certname|
        unless csr_exists?(certname)
          result[certname] = {
            :result  => 'fail',
            :message => "#{certname} could not be accepted because its certificate " +
                        "request has been moved or modified."
          }
        else
          api_response = @logger.log_action(user(env), certname, "Add") do
            CertAPI.add(certname)
          end
          if is_successful?(api_response.code)
            add_to_cache certname, CertAPI.cert_info(certname)
            result[certname] = {
              :result => 'success'
            }
          else
            result[certname] = {
              :result  => 'fail',
              :message => "A pending CSR for #{certname} was found, but the CA failed to sign it." +
                          " The CA responded with the message '#{api_response.body}'"
            }
          end
        end
      end
      result.to_json
    end

    # reject a CSR
    delete '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 403, "#{certname} could not be rejected because its certificate " +
                  "request has been moved or modified."
      else
        # The caching case for the reject/remove is slightly more complicated
        # than the other cases because once the csr has been rejected, you
        # can't request its state after rejected. To get around that we have
        # to make a temporary version of the csr, set its state to rejected
        # and then, if the call to reject succeeds, store that temporary
        # version in the cache
        cert = CertAPI.cert_info(certname)
        cert['state'] = 'rejected'
        api_response = @logger.log_action(user(env), certname, "Remove") do
          CertAPI.remove(certname)
        end
        unless is_successful?(api_response.code)
          halt 403, "A pending CSR for #{certname} was found, but the CA failed to sign it." +
                    " The CA responded with the message '#{api_response.body}'"
        end
        add_to_cache(certname, cert)
        api_response.code
      end
    end

    # Remove the actioned (approved/rejected) cert from the cache
    # After we approve or reject a CSR, it gets added to the cache so that the
    # users can see the state of it for up to 24 hours after approval or
    # rejection. A user may will to explicitly clear the cert from the cache
    # prior to cache expiration because it's cluttering the display
    post '/api/v1/cache/multi_purge' do
      result = {}

      params[:names].each do |certname|
        # We're logging because this action will affect the display for other users
        response_code = @logger.log_action(user(env), certname, "Cache purge") do
          purge_from_cache(certname) ? 200 : 500
        end

        if is_successful?(response_code)
          result[certname] = {:result => 'success'}
        else
          result[certname] = {:result => 'fail'}
        end
      end

      result.to_json
    end

    # reject multiple CSRs
    post '/api/v1/csrs/multi_reject' do
      content_type :json
      result = {}
      params[:names].each do |certname|
        unless csr_exists?(certname)
          result[certname] = {
            :result  => 'fail',
            :message => "#{certname} could not be rejected because its certificate " +
                        "request has been moved or modified."
          }
        else
          cert = CertAPI.cert_info(certname)
          cert['state'] = 'rejected'
          api_response = @logger.log_action(user(env), certname, "Remove") do
            CertAPI.remove(certname)
          end
          if is_successful?(api_response.code)
            add_to_cache(certname, cert)
            result[certname] = {
              :result => 'success'
            }
          else
            result[certname] = {
              :result  => 'fail',
              :message => "A pending CSR for #{certname} was found, but the CA failed to reject it." +
                          " The CA responded with the message '#{api_response.body}'"
            }
          end
        end
      end
      result.to_json
    end
  end
end
